import { Component, OnInit } from '@angular/core';
import { AgentBalancesService } from '../services/agent-balance.service';

@Component({
  selector: 'app-agent-balances',
  templateUrl: './agent-balances.component.html',
  styleUrls: ['./agent-balances.component.css'],
})
export class AgentBalancesComponent implements OnInit {
  countryList: any = [];
  selectedCountryList: any;
  agentList: any = [];
  selectedAgentList: any;
  agentBalances: any[] = [];
  fromDate: any = new Date(new Date().toLocaleString("en-US", {timeZone: "Asia/Riyadh"})).toISOString().substring(0, 10);
  toDate: any = new Date(new Date().toLocaleString("en-US", {timeZone: "Asia/Riyadh"})).toISOString().substring(0, 10);
  // fromDate: string = '';
  // toDate: string = '';
  isLoading: boolean = false;

  constructor(private agentBalancesService: AgentBalancesService) {}

  ngOnInit(): void {
    this.initializeDates();

    this.getCountryList();
  }

  initializeDates(): void {
    const today = new Date();
    const yyyy = today.getFullYear();
    const mm = String(today.getMonth() + 1).padStart(2, '0'); // Months are zero-based
    const dd = String(today.getDate()).padStart(2, '0');

    this.fromDate = `${yyyy}-${mm}-${dd}`;
    this.toDate = `${yyyy}-${mm}-${dd}`;
  }

  getCountryList() {
    this.agentBalancesService.getCountryList().subscribe({
      next: (res: any) => {
        this.countryList = res;
        this.selectedCountryList = res?.[0]?.CountryID;
        this.onCountryChange(this.selectedCountryList);
      },
      error: () => {
        this.countryList = [];
      },
    });
  }
  toggleSideBar() {
    const bodyElement = document.body;
    bodyElement.classList.toggle('sidebar-collapsein');
  }

  onCountryChange(countryId: any) {
    if (countryId != 'All') {
      this.agentBalancesService.getAgentList(countryId).subscribe({
        next: (res: any) => {
          this.agentList = res;
        },
        error: () => {
          this.agentList = [];
        },
      });
    }
  }
  displayByDate() {
    if (
      !this.fromDate ||
      !this.toDate ||
      !this.selectedCountryList ||
      !this.selectedAgentList
    ) {
      alert('Please select From Date, To Date, Country, and Agent.');
      return;
    }
    this.isLoading = true;
    const formattedFromDate = this.formatDate(this.fromDate);
    const formattedToDate = this.formatDate(this.toDate);

    this.agentBalancesService
      .getAgentBalancesGrid(
        this.selectedCountryList,
        this.selectedAgentList,
        formattedFromDate,
        formattedToDate
      )
      .subscribe({
        next: (res: any) => {
          this.agentBalances = res;
          this.isLoading = false;
        },
        error: (error) => {
          console.error(error);
          this.isLoading = false;
        },
      });
  }

  formatDate(date: string): string {
    const [year, month, day] = date.split('-');
    return `${year}-${month.padStart(2, '0')}-${day.padStart(2, '0')}`;
  }

  print() {
    window.print();
  }
}
