import { Component, OnInit } from '@angular/core';
import { AgentReportsService } from '../services/agent-reports.service';

@Component({
  selector: 'app-agent-reports',
  templateUrl: './agent-reports.component.html',
  styleUrls: ['./agent-reports.component.css'],
})
export class AgentReportsComponent implements OnInit {
  reports: string[] = [];
  countries: string[] = ['All'];
  currencies: any = [];
  agents: any = [];
  agentBranches: any = [];
  users: any = [];
  selectedReport: any;
  selectedCountry: any;
  selectedAgent: any;
  selectedCurrencies: any;
  selectedAgentBranch: any;
  selectedUser: any;
  isLoading: boolean = false;

  countryType: any = [
    {
      id: 'Send',
      value: 'Send',
    },
    {
      id: 'Receive',
      value: 'Receive',
    },
  ];
  selectedCountryType: any = '0';
  sendReceiveTransactions: any = [];
  fromDate: string;
  toDate: string;
  agentTransactionSummary: any = [];
  exchangeEarningReport: any = [];

  constructor(private agentReportsService: AgentReportsService) {}

  ngOnInit(): void {
    this.initializeDates();
    this.getAgentReport();
    this.getCountries('0');
  }

  initializeDates(): void {
    const today = new Date();
    const yyyy = today.getFullYear();
    const mm = String(today.getMonth() + 1).padStart(2, '0'); // Months are zero-based
    const dd = String(today.getDate()).padStart(2, '0');

    this.fromDate = `${yyyy}-${mm}-${dd}`;
    this.toDate = `${yyyy}-${mm}-${dd}`;
  }

  toggleSideBar() {
    const bodyElement = document.body;
    bodyElement.classList.toggle('sidebar-collapsein');
  }

  getAgentReport() {
    this.agentReportsService.getAgentReport().subscribe(
      (data: any[]) => {
        this.reports = data;
        this.selectedReport = data?.[0]?.Reports;
      },
      (error) => {
        console.error('Error fetching agent report:', error);
      }
    );
  }

  getCountries(countryType: string) {
    this.agentReportsService.getCountries(countryType).subscribe(
      (res: any) => {
        this.countries = res;
        this.selectedCountry = res?.[0]?.CountryID;
        this.getAgentList(this.selectedCountry);
      },
      (error) => {
        console.error('Error fetching agent report country:', error);
      }
    );
  }

  getAgentReportCurrency(agentId = 0) {
    this.agentReportsService.getAgentReportCurrency(agentId).subscribe(
      (res: any) => {
        this.currencies = res;
        this.selectedCurrencies = res?.[0]?.Currency_id;
      },
      (error) => {
        console.error('Error fetching agent report currency:', error);
      }
    );
  }

  getAgentList(countryId) {
    this.agentReportsService.getAgents(countryId).subscribe(
      (res: any) => {
        if (res?.length == 0) {
          alert('No data found.');
          return;
        }
        this.agents = res;
        this.agents =
          res.length > 0 ? res : [{ Agent_Name: 'Agent Not Found' }];
        this.selectedAgent = res?.[0]?.Agent_ID;
        if (this.selectedAgent) {
          this.getAgentBranchList(this.selectedAgent);
        } else {
          this.getAgentBranchList(0);
        }
        // this.getAgentReportCurrency(this.selectedAgent);
        // this.getUserList(this.selectedAgent);
      },
      (error) => {}
    );
  }

  onAgentChange() {
    if (this.selectedAgent) {
      this.getAgentBranchList(this.selectedAgent);
    }
  }

  onCountryChange() {
    if (this.selectedCountry) {
      this.getAgentList(this.selectedCountry);
    }
  }

  getAgentBranchList(agentId) {
    this.agentReportsService.getAgentBranches(agentId).subscribe(
      (res: any) => {
        this.agentBranches = res;
        this.selectedAgentBranch = res?.[0]?.AgentBranch_ID;
        this.agentBranches =
          res.length > 0
            ? res
            : [{ AgentBranch_Name: 'Agent Branch Not Found' }];
      },
      (error) => {}
    );
  }

  getUserList(agentId) {
    this.agentReportsService.getUserList(agentId).subscribe(
      (res: any) => {
        this.users = res;
        this.selectedUser = res?.[0]?.UserId;
      },
      (error) => {}
    );
  }

  onCountryTypeChange(countryType: string) {
    if (countryType != '0') {
      this.selectedCountryType = countryType;
      this.getCountries(countryType);
    }
  }

  getSendReceiveTransaction(agentId = 0, fromDate = '', toDate = '') {
    this.agentReportsService
      .getSendReceiveTransaction(agentId, fromDate, toDate)
      .subscribe(
        (res: any) => {
          if (res?.length == 0) {
            alert('No data found.');
            return;
          }
          this.sendReceiveTransactions = res;
        },
        (error) => {}
      );
  }

  getAgentTransactionSummary(
    agentId = 0,
    agentBranchId = 0,
    fromDate = '',
    toDate = ''
  ) {
    this.agentReportsService
      .getAgentTransactionSummary(agentId, agentBranchId, fromDate, toDate)
      .subscribe(
        (res: any) => {
          if (res?.length == 0) {
            alert('No data found.');
            return;
          }
          this.agentTransactionSummary = res;
        },
        (error) => {}
      );
  }

  getExchangeEarningReport(
    agentId = 0,
    agentBranchId = 0,
    fromDate = '',
    toDate = ''
  ) {
    this.agentReportsService
      .getExchangeEarningReport(agentId, agentBranchId, fromDate, toDate)
      .subscribe(
        (res: any) => {
          if (res?.length == 0) {
            alert('No data found.');
            return;
          }
          this.exchangeEarningReport = res;
        },
        (error) => {}
      );
  }

  async viewReport() {
    this.isLoading = true;

    try {
      if (this.selectedReport == 'Send/Receive Transaction') {
        if (!this.fromDate) {
          alert('Please Select From Date.');
          return;
        }
        if (!this.toDate) {
          alert('Please Select To Date.');
          return;
        }
        if (!this.selectedAgent) {
          alert('Please Select Agent.');
          return;
        }
      }
      if (
        this.selectedReport == 'Send/Receive Transaction' &&
        this.fromDate &&
        this.toDate &&
        this.selectedAgent
      ) {
        await this.getSendReceiveTransaction(
          this.selectedAgent,
          this.fromDate,
          this.toDate
        );
      }

      if (this.selectedReport == 'Agent Transaction Summary') {
        if (!this.fromDate) {
          alert('Please Select From Date.');
          return;
        }
        if (!this.toDate) {
          alert('Please Select To Date.');
          return;
        }
        if (!this.selectedAgent) {
          alert('Please Select Agent.');
          return;
        }

        if (!this.selectedAgentBranch) {
          alert('Please Select Agent Branch.');
          return;
        }
      }

      if (
        this.selectedReport == 'Agent Transaction Summary' &&
        this.fromDate &&
        this.toDate &&
        this.selectedAgent &&
        this.selectedAgentBranch
      ) {
        await this.getAgentTransactionSummary(
          this.selectedAgent,
          this.selectedAgentBranch,
          this.fromDate,
          this.toDate
        );
      }

      if (this.selectedReport == 'Exchange Earnings Report') {
        if (!this.fromDate) {
          alert('Please Select From Date.');
          return;
        }
        if (!this.toDate) {
          alert('Please Select To Date.');
          return;
        }
        if (!this.selectedAgent) {
          alert('Please Select Agent.');
          return;
        }

        if (!this.selectedAgentBranch) {
          alert('Please Select Agent Branch.');
          return;
        }
      }

      if (
        this.selectedReport == 'Exchange Earnings Report' &&
        this.fromDate &&
        this.toDate &&
        this.selectedAgent &&
        this.selectedAgentBranch
      ) {
        await this.getExchangeEarningReport(
          this.selectedAgent,
          this.selectedAgentBranch,
          this.fromDate,
          this.toDate
        );
      }
    } finally {
      this.isLoading = false;
    }
  }
}
