import { Component, ElementRef, ViewChild } from '@angular/core';
import { AllSentTransactionsService } from '../services/all-sent-transactions.service';
import * as XLSX from 'xlsx';

@Component({
  selector: 'app-all-sent-transactions',
  templateUrl: './all-sent-transactions.component.html',
  styleUrls: ['./all-sent-transactions.component.css'],
})
export class AllSentTransactionsComponent {
  @ViewChild('transactionTable') transactionTable: ElementRef;

  countryList: any = [];
  selectedCountryList: any;
  agentList: any = [];
  selectedAgentList: any = -1;
  agentBranchList: any = [];
  selectedAgentBranchList: any = 'ALL';

  // startDate: any;
  // endDate: any;
  startDate: any = new Date(new Date().toLocaleString("en-US", {timeZone: "Asia/Riyadh"})).toISOString().substring(0, 10);
  endDate: any = new Date(new Date().toLocaleString("en-US", {timeZone: "Asia/Riyadh"})).toISOString().substring(0, 10);
  isLoading: boolean = false;

  allSentTransaction: any = [];
  agentName: any = '';
  totalPayInAmt: any = '';
  totalCommission: any = '';
  totalVat: any = '';
  selectedFileFormat: any = '1';

  constructor(private transactionSvc: AllSentTransactionsService) {}

  ngOnInit(): void {
    this.initializeDates();
    this.getCountryList();
  }


  initializeDates(): void {
    const today = new Date();
    const yyyy = today.getFullYear();
    const mm = String(today.getMonth() + 1).padStart(2, '0'); // Months are zero-based
    const dd = String(today.getDate()).padStart(2, '0');

    this.startDate = `${yyyy}-${mm}-${dd}`;
    this.endDate = `${yyyy}-${mm}-${dd}`;
  }
 
  getCountryList() {
    this.transactionSvc.getCountryList().subscribe({
      next: (res: any) => {
        this.countryList = res;
        this.selectedCountryList = res?.[0]?.CountryID;
      },
      error: () => {
        this.countryList = [];
      },
      complete: () => {},
    });
  }

  onCountryChange(countryId: any) {
    if (countryId != 'All') {
      this.getAgentList(countryId);
    }
  }

  getAgentList(countryId: any = 0) {
    this.transactionSvc.getAgentList(countryId).subscribe({
      next: (res: any) => {
        this.agentList = res;
      },
      error: () => {
        this.agentList = [];
      },
      complete: () => {},
    });
  }

  onAgentChange(agentId: any) {
    if (agentId != 'All') {
      this.getAgentBranchList(agentId);
    }
  }

  getAgentBranchList(agentId: any = 0) {
    this.transactionSvc.getAgentBranchList(agentId).subscribe({
      next: (res: any) => {
        this.agentBranchList = res;
      },
      error: () => {
        this.agentBranchList = [];
      },
      complete: () => {},
    });
  }

  toggleSideBar() {
    const bodyElement = document.body;
    bodyElement.classList.toggle('sidebar-collapsein');
  }

  displayByDate() {
    if (!this.startDate && !this.endDate) {
      alert('Please Select Start & End Date!');
      return;
    }

    if (!this.startDate) {
      alert('Please Select Start Date!');
      return;
    }

    if (!this.endDate) {
      alert('Please Select End Date!');
      return;
    }

    if (!this.selectedCountryList) {
      alert('Please Select Country!');
      return;
    }
    this.isLoading = true;

    let payload: any = {};

    payload = {
      FromDate: this.startDate,
      ToDate: this.endDate,
      CountryID: this.selectedCountryList,
      AgentID: this.selectedAgentList ? this.selectedAgentList : -1,
      AgentBranchID: this.selectedAgentBranchList
        ? this.selectedAgentBranchList
        : 'All',
      UserGroupType: 0,
      SessionAgent_ID: 0,
    };
    this.transactionSvc.getFillGrid(payload).subscribe({
      next: (res: any) => {
        if (res?.length == 0) {
          alert('No data found.');
          return;
        }
        this.allSentTransaction = res;
        this.totalPayInAmt = this.allSentTransaction.reduce((total, snt) => total + (snt.PayInAmount || 0), 0);
        this.totalVat = res?.[0]?.Tot_VAT;
        this.totalCommission = res?.[0]?.Tot_Comm;
      },
      error: () => {
        this.allSentTransaction = [];
        this.isLoading = false;
      },
      complete: () => {
        this.isLoading = false;
      },
    });
  }

  get agent() {
    let agentName = '--All--';
    const agent = this.agentList.find(
      (a) => a.Agent_ID == this.selectedAgentList
    )?.Agent_Name;
    if (agent) {
      agentName = agent;
    }
    return agentName;
  }

  get totalNoofTrxns(): number {
    return this.allSentTransaction.length;
  }

  exportToExcel() {
    /* generate worksheet */
    const ws: XLSX.WorkSheet = XLSX.utils.table_to_sheet(
      this.transactionTable.nativeElement
    );

    /* generate workbook and add the worksheet */
    const wb: XLSX.WorkBook = XLSX.utils.book_new();
    XLSX.utils.book_append_sheet(wb, ws, 'Sheet1');

    /* save to file */
    const fileName = `Agentwise_Transaction_Sent${Date.now()}.xlsx`;
    XLSX.writeFile(wb, fileName);
  }
}
