import { Component, TemplateRef, ViewChild } from '@angular/core';
import { BeneficiaryRegistrationCheckerService } from '../beneficiary-registration-checker.service';
import { BsModalRef, BsModalService } from 'ngx-bootstrap/modal';
import { AuthService } from '../services/auth.service';
import { getCacheData } from '../utils/helper';
import { ToastrService } from 'ngx-toastr';

@Component({
  selector: 'app-beneficiary-registration-checker',
  templateUrl: './beneficiary-registration-checker.component.html',
  styleUrls: ['./beneficiary-registration-checker.component.css'],
})
export class BeneficiaryRegistrationCheckerComponent {
  @ViewChild('customerBeneficiaryChecker') customerBeneficiaryChecker;
  bsModalRef: BsModalRef;

  constructor(
    private modalService: BsModalService,
    private BeneficiaryRegistrationCheckerSvc: BeneficiaryRegistrationCheckerService,
    private authSvc: AuthService,
    private toastrSvc: ToastrService
  ) { }
  selectedCriteria: any = 1;
  customerDetails: any;
  viewedCustomer: any;
  beneficiaryNumber: string = '';
  nickName: string = '';
  startDate: string = '';
  endDate: string = '';
  allData: any = [];
  isLoading: boolean = false;

  ngOnInit() {
    this.startDate = this.authSvc.getTodayDate();
    this.endDate = this.authSvc.getTodayDate();
  }

  search() {
    if (this.selectedCriteria == 1 && !this.beneficiaryNumber) {
      alert('Please Enter Beneficiary No.');
      return;
    }

    if (this.selectedCriteria == 2 && !this.nickName) {
      alert('Please Enter Nick Name.');
      return;
    }

    if (this.selectedCriteria == 3 && !this.startDate && !this.endDate) {
      alert('Please Select Start and End Dates.');
      return;
    }
    this.BeneficiaryRegistrationCheckerSvc.getCustomer(
      this.selectedCriteria,
      this.startDate,
      this.endDate,
      this.beneficiaryNumber || this.nickName
    ).subscribe((res: any) => {
      if (res?.length == 0) {
        alert('No data found.');
        return;
      }
      this.allData = res;
    });
  }

  view(item) {
    this.BeneficiaryRegistrationCheckerSvc.viewCustomer(
      item?.BeneficiaryNo
    ).subscribe((res) => {
      this.viewedCustomer = res?.[0];
      const resData = res?.[0]; // Get the first item from the response array
      if (resData) {
        // Extract relevant properties from resData
        const {
          BeneficiaryNo,
          FirstName,
          MiddleName,
          LastName,
          NickName,
          Address,
          DateOfBirth,
          Nationality,
          MobileNumber,
          IDNumber,
          IDType,
          PayOutCountry,
          Currency,
          PaymentMode,
          PaymentBranchCode,
          AccountNumber,
          BankName,
          BranchName,
          BankCountry,
          BankCode,
          IsAuth,
          CustomerNo,
          RelationshipWithCustomer,
        } = resData;

        // Create an object with the extracted properties
        const customerDetailsObj = {
          BeneficiaryNo,
          FirstName,
          MiddleName,
          LastName,
          NickName,
          Address,
          DateOfBirth,
          Nationality,
          MobileNumber,
          IDNumber,
          IDType,
          PayOutCountry,
          Currency,
          PaymentMode,
          PaymentBranchCode,
          AccountNumber,
          BankName,
          BranchName,
          BankCountry,
          BankCode,
          IsAuth,
          CustomerNo,
          RelationshipWithCustomer,
        };

        // Map the object properties to label-value pairs
        this.customerDetails = Object.entries(customerDetailsObj).map(
          ([key, value]) => ({ label: key, value })
        );
      }
    });

    this.openModal(this.customerBeneficiaryChecker);
  }

  openModal(template: TemplateRef<any>) {
    this.bsModalRef = this.modalService.show(template, { class: 'modal-lg' });
  }

  reject() {
    this.BeneficiaryRegistrationCheckerSvc.reject(
      this.viewedCustomer.BeneficiaryNo,
      getCacheData('G_UserID'),
      this.viewedCustomer?.IsAuth
    ).subscribe({
      next: (res: any) => {
        if (res) {
          this.bsModalRef.hide();
          // alert('Transaction has been rejected.');
          this.toastrSvc.success('Transaction has been rejected.');
          return;
        }
      },
      error: (err: any) => {
        // alert(err?.error?.message);
        this.bsModalRef.hide();
        this.toastrSvc.error(err?.error?.message);
      },
      complete: () => {
        this.bsModalRef.hide();

      },
    });
  }

  Reject(item) {
    this.BeneficiaryRegistrationCheckerSvc.reject(
      item?.BeneficiaryNo,
      getCacheData('G_UserID'),
      item?.IsAuth
    ).subscribe({
      next: (res: any) => {
        if (res) {
          this.search();
          // alert('Transaction has been rejected.');
          this.toastrSvc.success('Transaction has been rejected.');
          return;
        }
      },
      error: (err: any) => {
        // alert(err?.error?.message);
        this.toastrSvc.error(err?.error?.message);
      },
      complete: () => {
      },
    });
  }

  ApproveTransaction() {
    // Filter the items that are checked
    const checkedItems = this.allData.filter((item) => item.isCheck);

    // Create the payload array for the selected items
    const payload = checkedItems.map((item) => ({
      IschkSelect: true,
      BeneficiaryNo: String(item.BeneficiaryNo),
      CustomerNo: String(item.CustomerNo),
      IsAuth: String(item.IsAuth),
      G_UserID: getCacheData('G_UserID'), // Assuming G_UserID is 0, adjust as necessary
    }));
    this.BeneficiaryRegistrationCheckerSvc.approve(payload)
      .subscribe({
        next: (res: any) => {
          if (res) {
            this.search();
            // alert('Records updated successfully.');
            this.toastrSvc.success('Records updated successfully.');
            return;
          }
        },
        error: (err: any) => {
          // alert(err?.error?.message);
          this.toastrSvc.error(err?.error?.message);
        },
        complete: () => {
        },
      });
  }

  approve() {
    const payload = [
      {
        IschkSelect: true,
        BeneficiaryNo: String(this.viewedCustomer?.BeneficiaryNo),
        CustomerNo: String(this.viewedCustomer?.CustomerNo),
        IsAuth: String(this.viewedCustomer?.IsAuth),
        G_UserID: getCacheData('G_UserID'),
      },
    ];
    this.BeneficiaryRegistrationCheckerSvc.approve(payload)
      .subscribe({
        next: (res: any) => {
          if (res) {
            this.bsModalRef.hide();
            // alert('Records updated successfully.');
            this.toastrSvc.success('Records updated successfully.');
            return;
          }
        },
        error: (err: any) => {
          // alert(err?.error?.message);
          this.bsModalRef.hide();
          this.toastrSvc.error(err?.error?.message);
        },
        complete: () => {
          this.bsModalRef.hide();
        },
      });
  }
  
  toggleSideBar() {
    const bodyElement = document.body;
    bodyElement.classList.toggle('sidebar-collapsein');
  }
}
