import { Component, TemplateRef, ViewChild } from '@angular/core';
import { BsModalRef, BsModalService } from 'ngx-bootstrap/modal';
import { CustomerRegistrationCheckerService } from '../services/customer-registration-checker.service';
import { AuthService } from '../services/auth.service';
import { getCacheData } from '../utils/helper';
import { ToastrService } from 'ngx-toastr';

@Component({
  selector: 'app-customer-registration-checker',
  templateUrl: './customer-registration-checker.component.html',
  styleUrls: ['./customer-registration-checker.component.css'],
})
export class CustomerRegistrationCheckerComponent {
  @ViewChild('customerRegistrationChecker') customerRegistrationCheckerModal;
  bsModalRef: BsModalRef;

  searchType: any = '1';
  search: any = '';
  fromDate: any = '';
  toDate: any = '';
  customerRegistrations: any = [];
  customerDetails: any = [];
  viewedCustomer: any;
  constructor(
    private customerRegistrationSvc: CustomerRegistrationCheckerService,
    private modalService: BsModalService,
    private authSvc: AuthService,
    private toastrSvc: ToastrService
  ) { }

  ngOnInit() {
    this.fromDate = this.authSvc.getTodayDate();
    this.toDate = this.authSvc.getTodayDate();
  }

  getAllRecord(searchType = '', search = '') {
    this.customerRegistrationSvc.getAllRecord(searchType, search).subscribe({
      next: (res: any) => {
        if (res?.length == 0) {
          alert('No data found.');
          return;
        }
        this.customerRegistrations = res;
      },
      error: (err: any) => {
        this.customerRegistrations = [];
        // Parse the error response and display a meaningful message
        if (err.error) {
          const errorMessage =
            err.error.message ||
            'Something went wrong while processing your request.';
          alert(errorMessage); // Using alert to show the error message
        } else {
          // Fallback error message if the error structure is not as expected
          alert('An unexpected error occurred.');
        }
      },
      complete: () => { },
    });
  }

  getAllRecordByDate(searchType = '', fromDate = '', toDate = '') {
    this.customerRegistrationSvc
      .getAllRecordByDate(searchType, fromDate, toDate)
      .subscribe({
        next: (res: any) => {
          if (res?.length == 0) {
            alert('No data found.');
            return;
          }
          this.customerRegistrations = res;
        },
        error: (err) => {
          this.customerRegistrations = [];
        },
        complete: () => { },
      });
  }

  searchData() {
    if (this.searchType == '1' && !this.search) {
      alert('Please Enter Data.');
      return;
    }

    if (this.searchType == '2' && !this.search) {
      alert('Please Enter Data.');
      return;
    }

    if ((this.searchType == '3' && !this.fromDate) || !this.toDate) {
      alert('Please Select From and To Dates.');
      return;
    }
    if (this.searchType == '1' || this.searchType == '2') {
      this.customerRegistrations = [];
      this.getAllRecord(this.searchType, this.search);
    }
    if (this.searchType == '3') {
      this.customerRegistrations = [];
      this.getAllRecordByDate(this.searchType, this.fromDate, this.toDate);
    }
  }

  onSelectChange() {
    this.customerRegistrations = [];
  }

  openModal(template: TemplateRef<any>) {
    this.bsModalRef = this.modalService.show(template, { class: 'modal-lg' });
  }

  approve() {
    this.customerRegistrationSvc
      .approve(283, this.viewedCustomer?.CustomerNo, getCacheData('G_UserID'))
      .subscribe({
        next: (res: any) => {
          this.bsModalRef.hide();
          this.searchData();
          this.toastrSvc.success(res);
        },
        error: (err: any) => {
          // alert(err?.error?.message);
          this.bsModalRef.hide();
          this.toastrSvc.error(err?.error?.message);
        },
        complete: () => {
          this.bsModalRef.hide();
        },
      });
  }

  reject() {
    this.customerRegistrationSvc
      .reject(this.viewedCustomer?.CustomerNo, 1, this.viewedCustomer?.IsAuth)
      .subscribe({
        next: (res: any) => {
          this.bsModalRef.hide();
          this.searchData();
          this.toastrSvc.success(res);
        },
        error: (err: any) => {
          // alert(err?.error?.message);
          this.bsModalRef.hide();
          this.toastrSvc.error(err?.error?.message);
        },
        complete: () => {
          this.bsModalRef.hide();
        },
      });
  }

  Reject(item) {
    this.customerRegistrationSvc
      .reject(item?.CustomerNo, 1, item?.IsAuth)
      .subscribe({
        next: (res: any) => {
          if (res) {
            this.searchData();
            this.toastrSvc.success('Records updated successfully.');
          }
        },
        error: (err: any) => {
          // alert(err?.error?.message);
          this.toastrSvc.error(err?.error?.message);
        },
        complete: () => {
        },
      });
  }

  update(item) {
    const payload = [
      {
        IschkSelect: true,
        CustomerNo: item?.CustomerName,
        G_USERID: getCacheData('G_UserID'),
      },
    ];

    this.customerRegistrationSvc.update(payload)
      .subscribe({
        next: (res: any) => {
          if (res) {
            this.toastrSvc.success(res);
          }
        },
        error: (err: any) => {
          // alert(err?.error?.message);
          this.toastrSvc.error(err?.error?.message);
        },
        complete: () => {
        },
      });
  }

  ApproveTransaction() {
    // Filter the items that are checked
    const checkedItems = this.customerRegistrations.filter(
      (item) => item.isCheck
    );

    // Create the payload array for the selected items
    const payload = checkedItems.map((item) => ({
      IschkSelect: true,
      CustomerNo: item?.CustomerName,
      G_USERID: getCacheData('G_UserID'),
    }));
    this.customerRegistrationSvc.update(payload)
      .subscribe({
        next: (res: any) => {
          if (res) {
            this.searchData();
            this.toastrSvc.success('Records updated successfully.');
          }
        },
        error: (err: any) => {
          // alert(err?.error?.message);
          this.toastrSvc.error(err?.error?.message);
        },
        complete: () => {
        },
      });
  }

  // viewCustomer(customer: any) {
  //   this.customerRegistrationSvc
  //     .viewCustomerById(customer?.IDNumber, 283)
  //     .subscribe({
  //       next: (res: any) => {
  //         this.viewedCustomer = res?.[0];
  //         console.log('res?.[0]: ', res?.[0]);
  //         const resData = res?.[0]; // Get the first item from the response array
  //         if (resData) {
  //           // Extract relevant properties from resData
  //           const {
  //             CustomerNo,
  //             FirstName,
  //             MiddleName,
  //             LastName,
  //             Address,
  //             Gender,
  //             DateOfBirth,
  //             Nationality,
  //             MobileNumber,
  //             IDType,
  //             IDNumber,
  //             IDExpiryDate,
  //             AccountNumber,
  //             SourceOfFund,
  //             TotalMonthlyIncome,
  //             Status,
  //             IsAuth,
  //           } = resData;

  //           // Create an object with the extracted properties
  //           const customerDetailsObj = {
  //             CustomerNo,
  //             FirstName,
  //             MiddleName,
  //             LastName,
  //             Address,
  //             Gender,
  //             DateOfBirth,
  //             Nationality,
  //             MobileNumber,
  //             IDType,
  //             IDNumber,
  //             IDExpiryDate,
  //             AccountNumber,
  //             SourceOfFund,
  //             TotalMonthlyIncome,
  //             Status,
  //             IsAuth,
  //           };

  //           // Map the object properties to label-value pairs
  //           this.customerDetails = Object.entries(customerDetailsObj).map(
  //             ([key, value]) => ({ label: key, value })
  //           );
  //         }
  //       },
  //       error: (err) => {
  //         this.customerRegistrations = [];
  //       },
  //       complete: () => {
  //         this.openModal(this.customerRegistrationCheckerModal);
  //       },
  //     });
  // }
  viewCustomer(customer: any) {
    console.log('Fetching details for customer ID:', customer?.IDNumber);
    this.customerRegistrationSvc
      .viewCustomerById(customer?.IDNumber, 283)
      .subscribe({
        next: (res: any) => {
          console.log('API Response:', res);
          if (res && res.length > 0) {
            this.viewedCustomer = res[0];
            console.log('Customer Details:', this.viewedCustomer);
            const resData = this.viewedCustomer;
            if (resData) {
              const {
                CustomerNo,
                FirstName,
                MiddleName,
                LastName,
                Address,
                Gender,
                DateOfBirth,
                Nationality,
                MobileNumber,
                IDType,
                IDNumber,
                IDExpiryDate,
                AccountNumber,
                SourceOfFund,
                TotalMonthlyIncome,
                Status,
                IsAuth,
              } = resData;

              const customerDetailsObj = {
                CustomerNo,
                FirstName,
                MiddleName,
                LastName,
                Address,
                Gender,
                DateOfBirth,
                Nationality,
                MobileNumber,
                IDType,
                IDNumber,
                IDExpiryDate,
                AccountNumber,
                SourceOfFund,
                TotalMonthlyIncome,
                Status,
                IsAuth,
              };

              this.customerDetails = Object.entries(customerDetailsObj).map(
                ([key, value]) => ({
                  label: key,
                  value,
                })
              );
            }
            this.openModal(this.customerRegistrationCheckerModal);
          } else {
            alert('No data found for the provided customer ID.');
          }
        },
        error: (err) => {
          console.error('Error fetching customer details:', err);
          alert('Failed to fetch customer details. Please try again later.');
        },
        complete: () => { },
      });
  }

  toggleSideBar() {
    const bodyElement = document.body;
    bodyElement.classList.toggle('sidebar-collapsein');
  }
}
