import { Component, TemplateRef, ViewChild } from '@angular/core';
import { DownloadTransactionsFileService } from '../download-transactions-file.service';
import { BsModalRef, BsModalService } from 'ngx-bootstrap/modal';
import * as XLSX from 'xlsx';

@Component({
  selector: 'app-download-transactions-file',
  templateUrl: './download-transactions-file.component.html',
  styleUrls: ['./download-transactions-file.component.css'],
})
export class DownloadTransactionsFileComponent {
  constructor(
    private modalService: BsModalService,
    private downTrnFileSvc: DownloadTransactionsFileService
  ) {}

  fileDetail: any;
  searchByType: any = '1';
  searchText: any;
  UserName: any;
  Data: any = [];
  isLoading: boolean = false;
  currentPage = 1;
  itemsPerPage = 15;
  selectedFileFormat: string = 'excel'; // Default selected format

  bsModalRef: BsModalRef;
  @ViewChild('viewLog') viewLogModal;

  ngOnInit() {
    this.getTransactionDownloadFile();
  }

  getTransactionDownloadFile() {
    this.isLoading = true;

    this.downTrnFileSvc.getTransactionDownloadFileList(0, 1).subscribe({
      next: (res: any) => {
        if (res?.length == 0) {
          alert('No data found.');
          return;
        }
        this.Data = this.removeAnchorTagFromArray(res);
      },
      complete: () => {
        this.isLoading = false;
      },
    });
  }

  removeAnchorTagFromArray(dataArray: any[]): any[] {
    return dataArray.map((item) => {
      if (item && item.PM_FILENM) {
        item.PM_FILENM = item.PM_FILENM.replace(/<a\s*.*?>|<\/a>/gi, '');
      }
      return item;
    });
  }

  toggleSideBar() {
    const bodyElement = document.body;
    bodyElement.classList.toggle('sidebar-collapsein');
  }

  SearchType(event) {
    this.searchByType = Number(event.target.value);
  }

  showAuditData() {
    if (this.searchText && this.searchByType) {
      this.downTrnFileSvc
        .getTransactionDownloadFileList(this.searchText, this.searchByType)
        .subscribe({
          next: (res: any) => {
            this.Data = res;
          },
          error: (err) => {
            this.Data = [];
          },
          complete: () => {},
        });
    } else {
      this.getTransactionDownloadFile();
    }
  }

  openModal(template: TemplateRef<any>) {
    this.bsModalRef = this.modalService.show(template);
  }

  // Add this function to your component
  getDisplayFilename(htmlString: string): string {
    const regex = /GetAgraniTxn\('(.+?)'\)/;
    const match = htmlString.match(regex);
    return match ? match[1] : htmlString;
  }

  // Assuming you fetch data from a service

  fileDetails(item) {
    this.downTrnFileSvc.fileDetails(item).subscribe((res) => {
      this.openModal(this.viewLogModal);
      this.fileDetail = res;
    });
  }

  firstPage() {
    this.currentPage = 1;
  }

  prevPage() {
    if (this.currentPage > 1) {
      this.currentPage--;
    }
  }

  nextPage() {
    const totalPages = this.getTotalPages();
    if (this.currentPage < totalPages) {
      this.currentPage++;
    }
  }

  lastPage() {
    const totalPages = this.getTotalPages();
    this.currentPage = totalPages;
  }

  get totalPage() {
    let total = 0;
    if (this.Data.length > 0) {
      total = this.getTotalPages();
    }
    return total;
  }

  getTotalPages() {
    return Math.ceil(this.Data.length / this.itemsPerPage);
  }

  getStartIndex() {
    return (this.currentPage - 1) * this.itemsPerPage;
  }

  getEndIndex() {
    return this.getStartIndex() + this.itemsPerPage;
  }

  getDataForCurrentPage(): any[] {
    return this.Data.slice(this.getStartIndex(), this.getEndIndex());
  }

  refresh() {
    this.getTransactionDownloadFile();
  }

  exportToExcel(): void {
    if (this.selectedFileFormat === 'excel') {
      const tableElement = document.getElementById('viewLogTable');
      if (!tableElement) {
        return;
      }

      const worksheet: XLSX.WorkSheet = XLSX.utils.table_to_sheet(tableElement);
      const workbook: XLSX.WorkBook = XLSX.utils.book_new();
      XLSX.utils.book_append_sheet(workbook, worksheet, 'View Log');
      XLSX.writeFile(workbook, 'ViewLogData.xlsx');
    }
    // Implement other export formats if needed
  }
}
