import { Component, OnInit } from '@angular/core';
import { SendRemittanceService } from '../services/send-remittance.service';
import { FormGroup, FormControl } from '@angular/forms';
import { ToastrService } from 'ngx-toastr';

@Component({
  selector: 'app-send-remittance',
  templateUrl: './send-remittance.component.html',
  styleUrls: ['./send-remittance.component.css'],
})
export class SendRemittanceComponent implements OnInit {
  agents: any[] = [];
  agentBranches: any[] = [];
  transferTypes: any[] = [];
  statuses: string[] = [];
  checkones: any;
  todateValue: any;
  fromdateValue: any;
  selectCountryType: any = '0';
  agentid: any;
  tabledata: any;
  selectedAgentName: any;
  agentBranchVAlue: any;
  allSentTransaction: any[] = []; // Variable to hold the response data

  selectCountry: any;
  transferTypeForm: FormGroup;

  constructor(private sendRemittanceService: SendRemittanceService,
    private toastrSvc: ToastrService

  ) { }

  ngOnInit() {
    this.loadTransferTypes();
    this.loadStatuses();

    // Get current date
    const currentDate = this.formatDate(new Date());

    const formattedCurrentDate = this.formatDate(currentDate);
    this.fromdateValue = formattedCurrentDate;
    this.todateValue = formattedCurrentDate;
    // Initialize the FormGroup with FormControls set to true and current date
    this.transferTypeForm = new FormGroup({
      fromDate: new FormControl(currentDate),
      toDate: new FormControl(currentDate),
      agentBranch: new FormControl('ALL'),
      bankTransfer: new FormControl(true),
      cashTransfer: new FormControl(true),
      draftTransfer: new FormControl(true),
      ComplianceWithdraw: new FormControl(true),
      Cancellationrefund: new FormControl(true),
      Processed: new FormControl(true),
      Refundcharges: new FormControl(true),
      Old: new FormControl(true),
      ComplianceQueue: new FormControl(true),
      Cancellationrefundcharges: new FormControl(true),
      TransactionRejected: new FormControl(true),
      CancellationRefundCharges: new FormControl(true),
      DoubtfulFailed: new FormControl(true),
      Approved: new FormControl(true),
      HoldDueInsufficient: new FormControl(true),
      Hold: new FormControl(true),
      PendingApproval: new FormControl(true),
      ComplianceHold: new FormControl(true),
      UnPaid: new FormControl(true),
      DownloadTransaction: new FormControl(true),
      Paid: new FormControl(true),
      PendingForAuthorization: new FormControl(true),
      ComplianceBlock: new FormControl(true),
      InProcess: new FormControl(true),
      AgentBlocked: new FormControl(true),
      Unclaimed: new FormControl(true),
      PendingAuthorization: new FormControl(true),
      Pending: new FormControl(true),
      Cancellationrequestispendingorinprocess: new FormControl(true),
    });
  }

  loadAgents(selectedType: string) {
    this.agentid = selectedType;

    this.sendRemittanceService.getAgents(selectedType).subscribe(
      (data: any[]) => {
        this.agents = data;

        if (this.agents.length > 0) {
          this.loadAgentBranches(this.agents[0].Agent_ID);
        }
      },
      (error) => {
        console.error('Error fetching agents:', error);
      }
    );
  }

  getAgentName(agentId: number): string | null {
    const agent = this.agents.find((agent) => agent.Agent_ID === agentId);
    return agent ? agent.Agent_Name : null;
  }

  loadAgentBranches(agentId: number) {
    this.selectedAgentName = this.getAgentName(agentId);

    this.sendRemittanceService.getAgentBranches(agentId).subscribe(
      (data: any[]) => {
        this.agentBranches = data;
      },
      (error) => {
        console.error('Error fetching agent branches:', error);
      }
    );
  }

  loadTransferTypes() {
    this.sendRemittanceService.getTransferTypes().subscribe(
      (data: any[]) => {
        // Store the transfer types to use in the template
        this.transferTypes = data;

        // Create a FormGroup dynamically with each transfer type as a FormControl
        const controls = data.reduce((acc, type) => {
          acc[type.Tr_Description] = new FormControl(true); // Initially checked
          return acc;
        }, {});

        // Assign the dynamically created controls to the form group
        this.transferTypeForm = new FormGroup(controls);
      },
      (error) => {
        console.error('Error fetching transfer types:', error);
      }
    );
  }

  toggleSideBar() {
    const bodyElement = document.body;
    bodyElement.classList.toggle('sidebar-collapsein');
  }

  loadStatuses() {
    this.sendRemittanceService.getStatus().subscribe(
      (data: any[]) => {
        const controls = data.reduce((acc, status) => {
          acc[status.Tr_Status_Name] = new FormControl(true); // Initially checked
          return acc;
        }, {});

        this.transferTypeForm = new FormGroup(controls);
        this.statuses = data.map((status) => status.Tr_Status_Name);
      },
      (error) => {
        console.error('Error fetching statuses:', error);
      }
    );
  }

  onCountryTypeChange(selectedType: string) {
    this.selectCountryType = selectedType;
    console.log('Selected Country Type:', selectedType);
    this.sendRemittanceService.countrySend(selectedType).subscribe(
      (data: any) => {
        this.selectCountry = data; // Store the received agent data
      },
      (error) => {
        console.error('Error fetching agent branches:', error);
      }
    );
  }

  todate(selectedType: string) {
    this.todateValue = this.formatDate(selectedType);
    console.log('Selected Country Type:', this.todateValue);
  }

  fromdate(selectedType: string) {
    console.log(selectedType);
    this.fromdateValue = this.formatDate(selectedType);
    console.log('Selected Country Type:', this.fromdateValue);
  }

  agentBranch(selectedType: string) {
    console.log(selectedType);
    this.agentBranchVAlue = selectedType;
    console.log('Selected Country Type:', this.agentBranchVAlue);
  }

  formatDate(date: Date | string): string {
    const d = new Date(date);
    const month = ('0' + (d.getMonth() + 1)).slice(-2); // months are zero-based
    const day = ('0' + d.getDate()).slice(-2);
    const year = d.getFullYear();
    return `${year}-${month}-${day}`;
  }

  viewReport() {
    const formValues = this.transferTypeForm.value;

    const payload = {
      FromDate: this.fromdateValue,
      ToDate: this.todateValue,
      CountryType: this.selectCountryType,
      chkTrxnType: true,
      chkStatus: true,
      AgentID: this.agentid,
      AgentBranchID: this.agentBranchVAlue,
      TrStatusIDs: this.statuses.map((status) => ({
        IsSelected: formValues[status],
        SelectedID: status,
      })),
      TrTypeIDs: this.transferTypes.map((type) => ({
        IsTrSelected: formValues[type.Tr_Description],
        TrSelectedID: type.Tr_Description,
      })),
    };

    console.log('Payload:', payload);
    // Call the service method to send the payload
    this.sendRemittanceService.grid(payload).subscribe(
      (response) => {
        if (response && Object.keys(response).length > 0) {
          this.tabledata = response;
          console.log('Report generated successfully:', response);
        } else {
          this.tabledata = []; // clear previous data if any
          console.log('No data found');
          alert('No data found');
        }
      },
      (error) => {
        console.error('Error generating report:', error);
        // alert(error.error.message);
        this.toastrSvc.error(error?.error?.message);

      }
    );
  }
}
