import { Component, ElementRef, ViewChild } from '@angular/core';
import * as XLSX from 'xlsx';
import { SentTransactionSummaryService } from '../services/sent-transaction-summary.service';
import { ViewReceiveTransactionService } from '../services/view-receive-transaction.service';
import { AuthService } from '../services/auth.service';

@Component({
  selector: 'app-sent-transaction-summary',
  templateUrl: './sent-transaction-summary.component.html',
  styleUrls: ['./sent-transaction-summary.component.css'],
})
export class SentTransactionSummaryComponent {
  @ViewChild('sentTransactionTable') sentTransactionTable: ElementRef;

  countryList: any = [];
  selectedCountry: any = 0;
  agentList: any = [];
  selectedAgent: any;
  receiveCountryList: any = [];
  selectedReceiveCountry: any = 0;
  fromDate: any = '';
  toDate: any = '';
  modeType: any = [
    {
      id: -1,
      name: 'All',
    },
    {
      id: 2,
      name: 'Cash',
    },
    {
      id: 1,
      name: 'Bank',
    },
  ];
  selectedModeType: any = -1;
  allSentTransactionSummary: any = [];
  isInvalidRange: boolean = false;

  constructor(
    private sentTransactionSvc: SentTransactionSummaryService,
    private viewReceiveTransactionSvc: ViewReceiveTransactionService,
    private authSvc: AuthService
  ) {}

  ngOnInit(): void {
    this.getCountryList();
    this.getAgentList(0);
    this.getReceiveCountryList();
    this.fromDate = this.authSvc.getTodayDate();
    this.toDate = this.authSvc.getTodayDate();
  }

  getCountryList() {
    this.sentTransactionSvc.getCountryList().subscribe({
      next: (res: any) => {
        this.countryList = res;
      },
      error: (err) => {
        this.countryList = [];
      },
      complete: () => {},
    });
  }

  onCountryChange(countryId: any) {
    if (countryId != 0) {
      this.getAgentList(countryId);
    }
  }

  getAgentList(countryId: any = 0) {
    this.sentTransactionSvc.getAgentList(countryId).subscribe({
      next: (res: any) => {
        this.agentList = res;
        this.selectedAgent = res?.[0]?.Agent_ID;
      },
      error: (err) => {
        this.agentList = [];
      },
      complete: () => {},
    });
  }

  getReceiveCountryList() {
    this.sentTransactionSvc.getReceiveCountryList().subscribe({
      next: (res: any) => {
        this.receiveCountryList = res;
      },
      error: (err) => {
        this.receiveCountryList = [];
      },
      complete: () => {},
    });
  }

  displayByDate() {
    if (!this.toDate && !this.fromDate) {
      alert('Please Select From & To Date!');
      return;
    }
    this.sentTransactionSvc
      .getAllRecord(
        this.fromDate,
        this.toDate,
        this.selectedModeType,
        this.selectedCountry,
        this.selectedAgent,
        this.selectedReceiveCountry
      )
      .subscribe({
        next: (res: any) => {
          if (res?.length == 0) {
            alert('No data found.');
            return;
          }
          const sanitizedDataArray = this.removeHtmlTagsFromArrayOfObjects(res);
          this.allSentTransactionSummary = sanitizedDataArray;
          if (this.allSentTransactionSummary?.length) {
            this.allSentTransactionSummary.forEach((item) => {
              if (typeof item === 'object' && item !== null) {
                item.isBold = false;
                item.isBold =
                  item.SendingAgent.toLowerCase().includes('country total') ||
                  item.ReceivingCountry.toLowerCase().includes('total');
              }
            });
          }
        },
        error: () => {
          this.allSentTransactionSummary = [];
        },
        complete: () => {},
      });
  }

  exportToExcel() {
    /* generate worksheet */
    const ws: XLSX.WorkSheet = XLSX.utils.table_to_sheet(
      this.sentTransactionTable.nativeElement
    );

    /* generate workbook and add the worksheet */
    const wb: XLSX.WorkBook = XLSX.utils.book_new();
    XLSX.utils.book_append_sheet(wb, ws, 'Sheet1');

    /* save to file */
    const fileName = `Sent_Transaction_Summary${Date.now()}.xlsx`;
    XLSX.writeFile(wb, fileName);
  }

  // onSelectDates() {
  //   try {
  //     const startDateObj = new Date(this.fromDate);
  //     const endDateObj = new Date(this.toDate);

  //     const sixMonthsBeforeStartDate = new Date(startDateObj);
  //     sixMonthsBeforeStartDate.setMonth(
  //       sixMonthsBeforeStartDate.getMonth() - 6
  //     );

  //     if (endDateObj.getTime() !== sixMonthsBeforeStartDate.getTime()) {
  //       this.isInvalidRange = true;
  //     }
  //   } catch (error) {}
  // }

  onSelectDates() {
    try {
      const startDateObj = new Date(this.fromDate);
      const endDateObj = new Date(this.toDate);

      // Calculate the date 6 months before the start date
      const sixMonthsBeforeStartDate = new Date(startDateObj);
      sixMonthsBeforeStartDate.setMonth(
        sixMonthsBeforeStartDate.getMonth() - 6
      );

      // Calculate the date 6 months after the start date
      const sixMonthsAfterStartDate = new Date(startDateObj);
      sixMonthsAfterStartDate.setMonth(sixMonthsAfterStartDate.getMonth() + 6);

      // Check if end date is within the 6-month range from start date
      if (
        endDateObj < sixMonthsBeforeStartDate ||
        endDateObj > sixMonthsAfterStartDate
      ) {
        this.isInvalidRange = true; // Invalid range
      } else {
        this.isInvalidRange = false; // Valid range
      }
    } catch (error) {
      this.isInvalidRange = true; // Handle errors by setting isInvalidRange to true
    }
  }

  removeHtmlTagsFromArrayOfObjects(arr: any[]): any[] {
    return arr.map((obj) => {
      const newObj: any = {};
      for (const key in obj) {
        if (Object.prototype.hasOwnProperty.call(obj, key)) {
          const value = obj[key];
          if (typeof value === 'string') {
            newObj[key] = value.replace(/<[^>]*>/g, ''); // Remove HTML tags using regex
          } else {
            newObj[key] = value;
          }
        }
      }
      return newObj;
    });
  }

  toggleSideBar() {
    const bodyElement = document.body;
    bodyElement.classList.toggle('sidebar-collapsein');
  }
}
