import { HttpClient, HttpHeaders } from '@angular/common/http';
import { Injectable } from '@angular/core';
import { environment } from 'src/environments/environment';
import { getCacheData } from '../utils/helper';

@Injectable({
  providedIn: 'root',
})
export class TagMappingService {
  url = environment.apiUrl;

  constructor(private http: HttpClient) {}

  getMappingMethodList() {
    return this.http.get(
      `${environment.apiUrl}/api/master/tag-mapping/mapping-method-list`,
      {
        headers: {
          Authorization: getCacheData('admin_token'),
          Accept: 'application/json',
          username: getCacheData('username'),
        },
      }
    );
  }

  getAgentList() {
    return this.http.get(
      `${environment.apiUrl}/api/master/tag-mapping/agent-list`,
      {
        headers: {
          Authorization: getCacheData('admin_token'),
          Accept: 'application/json',
          username: getCacheData('username'),
        },
      }
    );
  }



  getCategoryList() {
    return this.http.get(`
${this.url}/api/master/tag-mapping/category-list`,
      {
        headers: {
          Authorization: getCacheData('admin_token'),
          Accept: 'application/json',
          username: getCacheData('username'),
        },
      }
    );
  }

  getAllRecords(mappingMethod: string, agentID: number, category: string, requirementID: number) {
    return this.http.get(`${this.url}/api/master/tag-mapping/all-record`, {
      params: {
        MappingMethod: mappingMethod,
        AgentID: agentID.toString(),
        Category: category,
        RequirementID: requirementID.toString()
      },
      headers: {
        Authorization: getCacheData('admin_token'),
        Accept: 'application/json',
        username: getCacheData('username'),
      },
    });
  }


  getTritemList(mappingMethod: string) {
    return this.http.get(`${this.url}/api/master/tag-mapping/tritem-list`, {
      params: {
        MappingMethod: mappingMethod
      },
      headers: {
        Authorization: getCacheData('admin_token'),
        Accept: 'application/json',
        username: getCacheData('username'),
      },
    });
  }
  Mapupdate(id: any) {
    return this.http.get(`${this.url}/api/master/tag-mapping/edit`, {
      params: {
        ID: id
      },
      headers: {
        Authorization: getCacheData('admin_token'),
        Accept: 'application/json',
        username: getCacheData('username'),
      },
    });
  }


  updateTag(mappingMethod: string, payoutXMLTagID: number, agentID: number, trTag: number) {
    const userID = getCacheData('G_UserID');
    const adminToken = getCacheData('admin_token');
    const username = getCacheData('username');

    // Construct the request URL
    const requestUrl = `${this.url}/api/master/tag-mapping/update-tag?MethodName=${mappingMethod}&PayOutXMLTagID=${payoutXMLTagID}&PayoutAgentID=${agentID}&UserID=${userID}&TRTag=${trTag}`;

    // Set the headers
    const headers = new HttpHeaders({
      'Authorization':adminToken, // Ensure the token is prefixed with 'Bearer '
      'Accept': 'application/json',
      'username': username
    });

    // Make the PATCH request
    return this.http.patch(requestUrl, null, { headers });
  }
}